@@ -1,4 +1,3 @@
-# pylint: disable=C0103
 """
 Aggregate and grouping results:
 https://docs.microsoft.com/en-us/powerapps/developer/data-platform/webapi/query-data-web-api#aggregate-and-grouping-results
@@ -6,18 +5,20 @@ https://docs.microsoft.com/en-us/powerapps/developer/data-platform/webapi/query-
 FetchXML aggregation documentation:
 https://docs.microsoft.com/en-us/powerapps/developer/data-platform/use-fetchxml-aggregation
 """
+from typing import Optional
 
 from .typing import FilterType, List, Literal
 
 __all__ = ["apl"]
 
 
-class apl:
+class apl:  # noqa: N801
     """Convenience functions for creating $apply parameters."""
 
     @staticmethod
-    def groupby(columns: List[str], aggregate: str = None) -> str:
-        """Group results by columns, optionally aggregate.
+    def groupby(columns: List[str], aggregate: Optional[str] = None) -> str:
+        """
+        Group results by columns, optionally aggregate.
 
         :param columns: Columns to group by.
         :param aggregate: Aggregate grouped results by this function. Use `apl.aggregate(...)` to construct this.
@@ -26,8 +27,9 @@ class apl:
         return f"groupby(({','.join(columns)}){aggregate})"
 
     @staticmethod
-    def aggregate(*, col_: str, with_: Literal["average", "sum", "min", "max", "count"], as_: str) -> str:  # noqa: F821
-        """Aggregate column with some aggregation function, and alias the result under some name.
+    def aggregate(*, col_: str, with_: Literal["average", "sum", "min", "max", "count"], as_: str) -> str:
+        """
+        Aggregate column with some aggregation function, and alias the result under some name.
 
         :param col_: Column to aggregate over.
         :param with_: How to aggregate the columns.
@@ -36,8 +38,9 @@ class apl:
         return f"aggregate({col_} with {with_} as {as_})"
 
     @staticmethod
-    def filter(by: FilterType, group_by_columns: List[str]) -> str:
-        """Group filtered values by columns.
+    def filter(by: FilterType, group_by_columns: List[str]) -> str:  # noqa: A003
+        """
+        Group filtered values by columns.
 
         :param by: Filter results by this filter string before applying grouping. Use `ftr` to construct this.
         :param group_by_columns: Columns to group by.
@@ -47,6 +50,7 @@ class apl:
         elif isinstance(by, list):
             filters = " and ".join([value.strip() for value in by])
         else:
-            raise TypeError("Filter by must be either a set or a list.")
+            msg = "Filter by must be either a set or a list."
+            raise TypeError(msg)
 
         return f"filter({filters})/" + apl.groupby(group_by_columns)
